"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("../../..");
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    require('dotenv').config({ path: './config/northwind.env' });
    yield __1.orm.init('./config/northwind.yaml');
}));
describe('Execute', () => {
    const data = { "a": { "id": 1 } };
    test('function abs', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: p.price * -1, result: round(abs(p.price * -1), 10) }))';
        const expected = [{ "name": "Chai", "source": -18, "result": 18 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function acos', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 0.25, result: round(acos(0.25), 10) }))';
        const expected = [{ "name": "Chai", "source": 0.25, "result": 1.3181160717 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function asin', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 0.25, result: round(asin(0.25), 10) }))';
        const expected = [{ "name": "Chai", "source": 0.25, "result": 0.2526802551 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function atan', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 0.25, result: round(atan(0.25), 10) }))';
        const expected = [{ "name": "Chai", "source": 0.25, "result": 0.2449786631 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function atan2', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 0.50, result: round(atan2(0.25, 1), 10) }))';
        const expected = [{ "name": "Chai", "source": 0.5, "result": 0.2449786631 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function ceil', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 25.75, result: round(ceil(25.75), 10) }))';
        const expected = [{ "name": "Chai", "source": 25.75, "result": 26 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function cos', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 2, result: round(cos(2), 10) }))';
        const expected = [{ "name": "Chai", "source": 2, "result": -0.4161468365 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function exp', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 1, result: round(exp(1), 10) }))';
        const expected = [{ "name": "Chai", "source": 1, "result": 2.7182818285 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function floor', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 25.75, result: round(floor(25.75), 10) }))';
        const expected = [{ "name": "Chai", "source": 25.75, "result": 25 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function ln', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 2, result: round(ln(2), 10) }))';
        const expected = [{ "name": "Chai", "source": 2, "result": 0.6931471806 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function log', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, m: 10, n: 20, result: round(log(10, 20), 10) }))';
        const expected = [{ "name": "Chai", "m": 10, "n": 20, "result": 1.3010299957 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function round', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 135.375, result: round(135.375, 2) }))';
        const expected = [{ "name": "Chai", "source": 135.375, "result": 135.38 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function sign', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 255.5, result: round(sign(255.5), 10) }))';
        const expected = [{ "name": "Chai", "source": 255.5, "result": 1 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function tan', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 1.75, result: round(tan(1.75), 10) }))';
        const expected = [{ "name": "Chai", "source": 1.75, "result": -5.5203799225 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
    test('function trunc', () => __awaiter(void 0, void 0, void 0, function* () {
        const expression = 'Products.filter(p => p.id === id).map(p => ({ name: p.name, source: 135.375, result: round(trunc(135.375, 2), 10) }))';
        const expected = [{ "name": "Chai", "source": 135.375, "result": 135.37 }];
        const MySQLResult = yield __1.orm.execute(expression, data, { stage: 'MySQL' });
        expect(expected).toEqual(MySQLResult);
        const MariaDBResult = yield __1.orm.execute(expression, data, { stage: 'MariaDB' });
        expect(expected).toEqual(MariaDBResult);
        const PostgreSQLResult = yield __1.orm.execute(expression, data, { stage: 'PostgreSQL' });
        expect(expected).toEqual(PostgreSQLResult);
        const SqlServerResult = yield __1.orm.execute(expression, data, { stage: 'SqlServer' });
        expect(expected).toEqual(SqlServerResult);
        const MongoDBResult = yield __1.orm.execute(expression, data, { stage: 'MongoDB' });
        expect(expected).toEqual(MongoDBResult);
        const OracleResult = yield __1.orm.execute(expression, data, { stage: 'Oracle' });
        expect(expected).toEqual(OracleResult);
    }));
});
//# sourceMappingURL=numeric.test.js.map